# do not import dios-stuff here


class OptsFields:
    """storage class for the keys in ``dios_options``"""

    """Set the number of rows and variables to display in a call that use
    ``__repr__`` or ``__str__`` like e.g. ``print(dios)`` do."""
    disp_max_rows = "disp_max_rows "
    disp_max_vars = "disp_max_vars"

    """
    none_plus: none or more can be given
    at_leat_one: accept if at least one column is present
    all_present: all given columns must be present
    all_plus: (pandas, bool-series style) all must be present but more can be given
    """
    col_indexing_method = "dios_to_dios_method"

    mixed_itype_warn_policy = "mixed_itype_policy"
    comparison_nan_policy = "comparison_nan_policy"


class Opts:
    none_plus = 'none_plus'
    at_least_one = 'at_least_one'
    all_present = 'all_present'

    itype_warn = 'warn'
    itype_err = 'err'
    itype_ignore = 'ignore'

    keep_nans = 'keep'
    nplike_nans = 'nplike_nans '


# set default values
dios_options = {
    OptsFields.disp_max_rows: 10,
    OptsFields.disp_max_vars: 4,
    OptsFields.col_indexing_method: Opts.none_plus,
    OptsFields.mixed_itype_warn_policy: Opts.itype_ignore,
    OptsFields.comparison_nan_policy: Opts.keep_nans,
}


def check_keys_by_policy(check, keys, policy):

    filtered = [k for k in check if k in keys]
    if policy == Opts.none_plus:
        pass

    elif policy == Opts.at_least_one:
        if not filtered:
            raise KeyError("Policy says: at least one key must be shared.")

    elif Opts.all_present:
        fail = set(filtered).symmetric_difference(set(check))
        if fail:
            raise KeyError(f"Unknown keys {fail}. Policy says: all given keys must be known.")
    else:
        raise ValueError(policy)

    return filtered
