from dios import *
from profiling import *
import pandas as pd
import datetime as dt
import numpy as np
import pytest

# pytestmark = pytest.mark.skip


v0 = 'var0'
v1 = 'var1'
v2 = 'var2'
v3 = 'var3'
v4 = 'var4'
v5 = 'var5'
v6 = 'var6'
v7 = 'var7'
v8 = 'var8'
v9 = 'var9'


def gen_series(rows, randomize=True):
    start = dt.datetime.strptime("2000-01-10 00:00:00", "%Y-%m-%d %H:%M:%S")
    if randomize:
        sec = np.random.randint(1, 10)
        rows = np.random.randint(int(rows * 0.5), rows * 2)
        start += pd.Timedelta(sec)
    else:
        sec = 10
    times = pd.date_range(periods=rows, start=start, freq=f'{sec}s')
    d = np.random.randint(1, 9, rows)
    return pd.Series(data=d, index=times)


def test_getitem():
    # prepare
    begin = dt.datetime.strptime("2000-01-10 00:00:00", "%Y-%m-%d %H:%M:%S")
    t0 = begin + pd.Timedelta('20s')
    t1 = t0 + pd.Timedelta('50s')
    dios_aligned = DictOfSeries()
    dios_rand = DictOfSeries()

    # fill
    vars = 10
    rows = 10
    for i in range(0, vars):
        dios_aligned[f'var{i}'] = gen_series(rows, randomize=False)
        dios_rand[f'var{i}'] = gen_series(rows, randomize=True)

    # testsets
    var = [v0, [], [v1], [v0, v0], [v0, v2]]
    tssl = [slice(None), slice(t0, None), slice(None, t1), slice(t0, t1), slice(t0, t0)]
    ts = [t0, [], [t0], [t0, t0], [t0, t1]]

    for v in var:
        print(v)
        dios_rand[v]

    # use aligned dios for time stamps instead of time-ranges
    for t in tssl:
        print(t)
        dios_rand[v]

    try:
        dios_aligned[v1, v2]
    except KeyError:
        pass

    try:
        dios_aligned[v1, v2, v3]
    except KeyError:
        pass


def test_setitem():
    # prepare
    begin = dt.datetime.strptime("2000-01-10 00:00:00", "%Y-%m-%d %H:%M:%S")
    t0 = begin + pd.Timedelta('30s')
    t1 = t0 + pd.Timedelta('50s')
    dios_aligned = DictOfSeries()
    dios_aligned.name = 'aligned'
    dios_rand = DictOfSeries()
    dios_rand.name = 'rand'

    # fill
    vars = 10
    rows = 100
    for i in range(0, vars):
        dios_aligned[f'var{i}'] = gen_series(rows, randomize=False)
        dios_rand[f'var{i}'] = gen_series(rows, randomize=True)

    # testsets
    keys = [v0, [v1], [v0, v0], [v0, v2]]
    tssl = [slice(None), slice(t0, None), slice(None, t1), slice(t0, t1), slice(t0, t0)]
    scalars = [1, 'deadbeef']
    l = list(np.random.randint(0, 100, rows))
    dios = [dios_aligned.copy(), dios_rand.copy()]
    for d in dios:
        d.name = 'src-dios'

    # assign scalars
    for val in scalars:
        for v in keys:
            print(v, '=', val)
            dios_rand[v] = val

    # assign scalars
    for val in scalars:
        for t in tssl:
            print(t, '=', val)
            dios_rand[t] = val

    # assign list
    for v in keys:
        print(v, '=', l)
        dios_aligned[v] = l

    # assign series
    for v in keys:
        print(v, '=', 'series')
        dios_aligned[v] = dios_aligned[v4]

    # assign dios
    for v in keys:
        for d in dios:
            print(f'{v} = dios[{v}]')
            dios_aligned[v] = d[v]

    # probkeys = [[], slice(v0, v0), ]
    # for v in probkeys:
    #     try:
    #         dios_aligned[v] = l
    #     except ValueError:
    #         pass
    #     else:
    #         raise AssertionError("should return an error")


def test_integrity():
    rows = 1000
    cols = 10
    df, _, _, dios = get_testset(1000, 10)

    v = var_prefix + str(np.random.randint(0, cols))
    t = find_index_range(dios)
    t0, t1 = gen_random_timestamps(*t)

    # originals
    dios_ = dios.copy(deep=False)
    df_ = df.copy()
    s_ = df[v].dropna()

    # identity
    assert (dios_ == dios).all().all()
    assert (dios_[v] == dios[v]).all().all()
    assert dios_ is not dios
    assert dios_[v] is not dios[v]

    # equal t0 df
    assert dios[v] is not df[v]
    assert (dios[v] == df[v].dropna()).all().all()

    # write
    dios = dios_.copy()
    s = s_.copy()
    dios[t0:t1] = 4
    s[t0:t1] = 4
    assert (dios[v] == s).all()

    # write all
    s = s_.copy()
    dios = dios_.copy()
    dios[v] = 111
    s[:] = 111
    assert (dios[v] == s).all()

    # multi variables - slice
    df = df_.copy()
    dios = dios_.copy()
    li = [v0, v1, v2]
    dios[t0:t1] = 222
    for x in li:
        s = df[x].dropna()
        s[t0:t1] = 222
        assert (dios[x] == s).all()
    # on all
    dios[t0:t1] = 222.111
    m = df.loc[t0:t1,:].notna()
    df[m] = 222.111
    for x in df:
        s = df[x].dropna()
        assert (dios[x] == s).all()

    # multi variables - list
    df = df_.copy()
    dios = dios_.copy()
    li = [v0, v5, v3, v9]
    dios[t0:t1] = 333
    for x in li:
        s = df[x].dropna()
        s[t0:t1] = 333
        assert (dios[x] == s).all()

    # dios to dios
    df = df_.copy()
    dios = dios_.copy()
    dios[v] = 444
    dios[v5] = dios[v] * 0.1
    s = df[v].dropna()
    s[:] = 444 * 0.1
    assert (dios[v5] == s).all()


def test_foreach():
    # dios.foreach() is alias for dios.pipe()

    dios = DictOfSeries()
    ser = pd.Series([1,4,5,6,3,5,5,3,2,1,13])
    dios['a'] = ser
    dios['b'] = ser * 4

    # return type of function: Series
    d = dios.pipe(pd.Series.astype, float)
    assert isinstance(d, DictOfSeries)
    assert not d['a'].dtype == ser.dtype
    assert (d['a'] == ser.astype(float)).all()

    # return type of function: scalar
    d = dios.pipe(min)
    assert isinstance(d, pd.Series)
    assert d['a'] == ser.min()
    assert d['b'] == ser.min() * 4

    # return type of function: scalar
    # but with squeeze
    d = dios.copy()
    del d['a']
    d = d.pipe(min)
    assert isinstance(d, np.int64)
    assert d == 4

    # test inplace
    d = dios.copy()
    d1 = dios.copy()
    d2 = d.pipe(pd.Series.drop_duplicates, inplace=False)
    # original didtn change
    assert d is not d1 and (d == d1).all()
    # tests on d2
    assert d2 is not d
    assert isinstance(d2, DictOfSeries)
    assert (d2['a'] == ser.copy().drop_duplicates()).all()
    assert (d2['b'] == (ser * 4 ).copy().drop_duplicates()).all()

    d3 = d.pipe(pd.Series.drop_duplicates, inplace=True)
    assert d3 is None
    # original did (!) change
    assert d is not d1
    try: d == d1
    except ValueError: pass;
    else: raise AssertionError
    # check if gone well..
    assert (d['a'] == ser.copy().drop_duplicates()).all()
    assert (d['b'] == (ser * 4 ).copy().drop_duplicates()).all()


def tmptest():
    # prepare
    begin = dt.datetime.strptime("2000-01-10 00:00:00", "%Y-%m-%d %H:%M:%S")
    t0 = begin + pd.Timedelta('20s')
    t1 = t0 + pd.Timedelta('50s')
    diosa = DictOfSeries()
    diosr = DictOfSeries()

    # fill
    vars = 10
    rows = 8
    for i in range(0, vars):
        diosa[f'v{i}'] = gen_series(rows, randomize=False)
        diosr[f'v{i}'] = gen_series(rows, randomize=True)

    diosa[[], slice(None)] = 99
    exit(9)

    print(diosa)
    d = diosa[t0:t1, v1]
    d = d * 1000
    print(d)
    print()
    print(diosa[t0:t1, v1])
    l1 = len(diosa[t0:t1, v1])
    l2 = len(d)
    print(l1, l2)
    diosa[t0:t1, v1] = d
    print(diosa[v1])
    exit(3)


if __name__ == '__main__':
    dios_options['disp_max_rows'] = 20

    # tmptest()
    test_getitem()
    test_setitem()
    test_integrity()
    test_foreach()
